/**
  lp_Si7021 
  Modified by Matthew Ford
  (c) 2018 Forward Computing and Control Pty Ltd
  from SparkFun Si7021 Temperature and Humidity Breakout
  **/
/*
 SparkFun Si7021 Temperature and Humidity Breakout 
 By: Joel Bartlett
 SparkFun Electronics
 Date: December 10, 2015
 
 This is an Arduino library for the Si7021 Temperature and Humidity Sensor Breakout
 
 This library is based on the following libraries:

 HTU21D Temperature / Humidity Sensor Library
 By: Nathan Seidle
 https://github.com/sparkfun/HTU21D_Breakout/tree/master/Libraries

 Arduino Si7010 relative humidity + temperature sensor
 By: Jakub Kaminski, 2014
 https://github.com/teoqba/ADDRESS

 This Library is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This Library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 For a copy of the GNU General Public License, see
 <http://www.gnu.org/licenses/>.
 */

#ifndef LP_SI7021_H
#define LP_SI7021_H

#include <Arduino.h>

/****************Si7021 Definitions***************************/

#define ADDRESS      0x40

#define TEMP_MEASURE_HOLD  0xE3
#define HUMD_MEASURE_HOLD  0xE5
#define TEMP_MEASURE_NOHOLD  0xF3
#define HUMD_MEASURE_NOHOLD  0xF5
#define TEMP_PREV   0xE0

#define WRITE_USER_REG  0xE6
#define READ_USER_REG  0xE7
#define SOFT_RESET  0xFE

#define HTRE        0x02
#define _BV(bit) (1 << (bit))

#define CRC_POLY 0x988000 // Shifted Polynomial for CRC check

// Error codes
#define I2C_TIMEOUT 	998
#define BAD_CRC		999


class lp_Si7021 {
public:
	// Constructor
	lp_Si7021();

	bool  begin();
  void  end(); // calls Wire.end();


	// Si7021 & HTU21D Public Functions
	void startRHandTemp();
	float readRH();
	float readLastRHTemp();
    uint16_t readRH_Raw();
    uint16_t readLastRHTemp_Raw();

	void startTemp();
	float readTemp();
	uint16_t readTemp_Raw();

    int RH_RawToInt(uint16_t RH_Code); // due to accuracy of RH% whole int is good enough
    float Temp_RawToFloat(uint16_t temp_Code);

	float CtoF(float tempC);
	void  heaterOn();
	void  heaterOff();
	void  changeResolution(uint8_t i);
	void  reset();
	uint8_t  checkID();


private:
	uint16_t getResult();
	void startMeasurement(uint8_t command);
	uint16_t getTempForLastRH();
	void     writeReg(uint8_t value);
	uint8_t  readReg();
};

#endif
